#! /usr/bin/perl
#
# airraid (airraid.pl)    - Create fake 802.11 Access Points
#                           with Atheros wireless cards and
#                           MadWifi drivers
#
#
# Based on fakeap by Black Alchemy Enterprises.
# This code is released under the GNU Public License.
# Modified under the terms of the GPL on 15 Jan 2007
#
# See http://www.pettingers.org/code/airraid.html for details
# --------------------------------------------------------------------
# Notes:
#
# Requires (currently) macchanger to effectively change the mac address.
# See: http://www.alobbs.com/macchanger/
# There may be other ways to change the mac but macchanger works well
# MadWifi is reportedly working on direct driver support for mac changes.

use strict;
use warnings;
use Getopt::Long;
use Time::HiRes;  # See comment below if you do not want to load HiRes

use vars
  qw( $sleep_opt $channel_opt $mac_opt $rmac_opt $essid_opt $words_opt
  $interface_opt $vendors_opt $wep_opt $power_opt $group_opt);

###########################################
#### User Defined Variables ####
###########################################


my $MAX_CHANNEL = 11;                              # North America
my $IWCONFIG    = "/sbin/iwconfig";                # location of iwconfig
my $IFCONFIG    = "/sbin/ifconfig";                # location of ifconfig
my $WLAN        = "/usr/local/bin/wlanconfig";     # location of wlanconfig
my $MACC        = "/usr/local/bin/macchanger";     # location of mac changer
my $BASE        = "wifi0";                         # Base MadWifi device

my @words = ( "Access Point", "tsunami", "host", "airport", "linksys",
              "Netgear", "Cisco", "Wireless", "2wire", "intel", "WLAN" );
my @vendors = ( "00:00:0C:", "00:00:CE:", "00:00:EF:" );
my @group_wep = ();
my @group_chan = ();

##########################################
#### No User Variables Below ####
##########################################

# catch ctrl-c
#
$SIG{INT} = \&int_catcher;

# int_catcher
#
# args: none
# rets: none
#
# Signal catcher.  Prints message and exits.

sub int_catcher {
    print "\n";
    print "-------------------------------------------------------------------------\n";
    print "Run complete\n\n";

    exit;

}

# usage
#
# args: none
# rets: none
#
# Prints arguments and exits

sub usage {
    print "Usage: airraid.pl --interface athX [--channel X] [--essid NAME] [--mac XX:XX...]\n";
    print "     [--rmac 0|1] [--words FILENAME] [--sleep N] [--vendors FILENAME]\n";
    print "     [--wep N] [--power N] [--gf FILENAME]\n\n";

    print "  --channel X     Use static channel X\n";
    print "  --essid NAME    Use static ESSID NAME\n";
    print "  --mac XX:XX...  Use static MAC address XX:...\n";
    print "  --rmac 0|1      Use (0)all-vendor random MAC [default] or (1)hard-code vendors list\n";
    print "  --words FILE    Use FILE to create ESSIDs \n";
    print "  --sleep N       Sleep N Ssec between changes, default 0.6\n";
    print "  --vendor FILE   Use FILE to define vendor MAC prefixes \n";
    print "  --wep N         Use WEP with probability N where 0 < N <= 1\n";
    print "  --power N       Vary Tx power between 1 and N.  In milliwatts\n";
    print "  --gf FILE       Use Group File FILE for grouped MAC, ESSID, channel, crypto\n";
    print "                      Note: Group File Option (--gf) is mutually exclusive of\n";
    print "                      _EVERY_ other option except sleep and power\n";
    print "\n";
}

# load_words
#
# args: none
# rets: none
#
# Load the user specified word list into @words

sub load_words {
    @words = ();

    open( my $FH, "<$words_opt" ) or die "Could not open $words_opt: $!\n";
    while ( my $line = <$FH> ) {
        chomp $line;
        push @words, $line;
    }
    close $FH;

    return;
}

# load_vendors
#
# args: none
# rets: none
#
# Loads vendor mac prefix file into @vendors

sub load_vendors {
    @vendors = ();

    open( my $FH, "<$vendors_opt" ) or die "Could not open $vendors_opt: $!\n";
    while ( my $line = <$FH> ) {
        chomp $line;
        $line =~ /^(\w\w:\w\w:\w\w)/;
        push @vendors, "$1:";
    }
    close $FH;

    return;
}

# load_groups
#
# args: none
# rets: none
#
# Load the user specified groups into @words and @vendors

sub load_groups {
    @words = ();
    @vendors = ();
my  @group = ();

    open( my $FH, "<$group_opt" ) or die "Could not open $group_opt: $!\n";
    while ( my $line = <$FH> ) {
        chomp $line;
        @group = split(/,/, $line);
        push @vendors, $group[0];
        push @words, $group[1];
        push @group_wep, $group[2];
        push @group_chan, $group[3];
    }
    close $FH;
    return;
}


# gen_essid
#
# args: none
# rets: scalar string $word
#
# Returns a string suitable for use as an ESSID.  Picks it randomly from
# @words

sub gen_essid {
    return $words[ int( rand $#words ) ];
}

# gen_channel
#
# args: none
# rets: scalar int $channel
#
# Returns a number between 1 and $MAX_CHANNEL

sub gen_channel {
    return int( rand $MAX_CHANNEL ) + 1;
}

# gen_power
#
# args: none
# rets: scalar int $channel
#
# Returns a number between 1 and $power_opt or Def if $power_opt is not set

sub gen_power {
    return int( rand $power_opt ) + 1 if $power_opt;
    return "Default";
}

# gen_mac
#
# args: none
# rets: none
#
# Returns a random MAC address with first three octets from @vendors
# last three random.

sub gen_mac {

    return sprintf(
        "%s%02X:%02X:%02X",
        $vendors[ int( rand $#vendors ) ],
        int( rand 256 ),
        int( rand 256 ),
        int( rand 256 )
    );

}

# pick_wep
#
# args: none
# rets: string Y|N
#
# Returns N if wep_opt is not set otherwise returns
# Y with probability $wep_opt

sub pick_wep {
    $wep_opt = 0 if not $wep_opt;

    return "Y" if ( rand(1) < $wep_opt );
    return "N";

}

################################################################
#
# Main
#

# Set defaults


print "airraid 0.1 - Atheros Fake Access Points\n";


GetOptions(
    "channel=i"   => \$channel_opt,
    "essid=s"     => \$essid_opt,
    "words=s"     => \$words_opt,
    "mac=s"       => \$mac_opt,
    "rmac=i"      => \$rmac_opt,
    "sleep=s"     => \$sleep_opt,
    "power=i"     => \$power_opt,
    "wep=s"       => \$wep_opt,
    "interface=s" => \$interface_opt,
    "vendors=s"   => \$vendors_opt,
    "gf=s"        => \$group_opt
);

usage() and exit if not $interface_opt;
# mutually exclusive option filters
usage() and exit if ( ($words_opt and ($essid_opt or $group_opt))
                     or ($essid_opt and $group_opt) );
usage() and exit if ( ($mac_opt and ($rmac_opt or $group_opt))
                     or ($rmac_opt and $group_opt) );


print "Using interface $interface_opt:\n";
print "Sleeping $sleep_opt sec\n"             if $sleep_opt;
print "Static channel $channel_opt\n"         if $channel_opt;
print "Static ESSID $essid_opt\n"             if $essid_opt;
print "Static MAC $mac_opt\n"                 if $mac_opt;
print "Generating fully random MAC\n"         if !$rmac_opt and !$group_opt;
print "Generating ESSIDs from $words_opt\n"   if $words_opt;
print "Using WEP with probability $wep_opt\n" if $wep_opt;
print "Vary Tx power up to $power_opt\n"      if $power_opt;
print "Using Group File $group_opt\n"         if $group_opt;

if (!$group_opt) {
   load_words() if $words_opt;
   print "Using $#words words for ESSID generation\n";

   load_vendors() if $vendors_opt;
   print "Using $#vendors vendors for MAC generation\n";
   }
else {
   load_groups();
   print "Using $#vendors groups for MAC and ESSID generation\n";
   }

print
  "-------------------------------------------------------------------------\n";

for ( my $i = 0 ; ; $i++ ) {
#    my $essid   = $essid_opt   || gen_essid();
#    my $channel = $channel_opt || gen_channel();
#    my $mac     = $mac_opt     || gen_mac();
    my $sleep   = $sleep_opt   || 0.6;
#    my $wep     = pick_wep();
    my $power   = gen_power();
    my $mac='';
    my $essid='';
    my $wep='';
    my $channel='';

if (!$group_opt) {
    $mac     = $mac_opt     || gen_mac();
    $essid   = $essid_opt   || gen_essid();
    $wep     = pick_wep();
    $channel = $channel_opt || gen_channel();
   }
else {
    my $beacon  = int( rand ($#vendors) );
    $mac     = $vendors[$beacon];
    $essid   = $words[$beacon];
    $wep     = $group_wep[$beacon];
    $channel = $group_chan[$beacon];
    }



################################

     system( $IFCONFIG, $interface_opt, "down" );

     system( $WLAN, "$interface_opt", "destroy" );

     system( "'$WLAN' '$interface_opt' create wlandev '$BASE' wlanmode ap -bssid >/dev/null 2>&1" );
     if ($group_opt or $rmac_opt) {
        system ( "'$MACC' -m '$mac' '$BASE' >/dev/null 2>&1" );
        }
     elsif (!$rmac_opt) {
        system( "'$MACC' -a '$BASE' >/dev/null 2>&1" );
        }
     else {
        system( "'$MACC' -m '$mac' '$BASE' >/dev/null 2>&1" );
        }

     system( $WLAN, "$interface_opt", "destroy" );

     system( "'$WLAN' '$interface_opt' create wlandev '$BASE' wlanmode ap -bssid >/dev/null 2>&1" );
     if ($group_opt or $rmac_opt) {
        system ( "'$MACC' -m '$mac' '$BASE' >/dev/null 2>&1" );
        }
     elsif (!$rmac_opt) {
        system( "'$MACC' -a '$BASE' >/dev/null 2>&1" );
        }
     else {
        system( "'$MACC' -m '$mac' '$BASE' >/dev/null 2>&1" );
        }


     system( $IWCONFIG, $interface_opt, "ESSID",   $essid );
     system( $IWCONFIG, $interface_opt, "channel", $channel );
     system( $IWCONFIG, $interface_opt, "txpower", $power . "mW" ) if $power_opt;

    if ($wep_opt or $group_opt) {
        system( $IWCONFIG, $interface_opt, "key", "off" )
          if $wep eq "N";
        system( $IWCONFIG, $interface_opt, "key", "s:airraid!" )
          if $wep eq "Y";
    }

     system( $IFCONFIG, $interface_opt, "up" );

    printf( "%i: ESSID=%-15s chan=%02i Pwr=%-3s WEP=%s MAC=%s    \r",
        $i, $essid, $channel, $power, $wep, $mac );

# Use for delays of whole seconds (Time::HiRes not installed)
#   sleep($sleep);
#
    Time::HiRes::sleep($sleep);
}

